package com.server.alb.controllers

import com.server.alb.models.*
import com.server.alb.repositories.*
import com.server.alb.services.BugService
import com.server.alb.services.ProjectService
import com.server.alb.services.UserService
import jakarta.servlet.http.HttpServletRequest
import jakarta.validation.Valid
import org.springframework.security.authentication.AnonymousAuthenticationToken
import org.springframework.security.core.Authentication
import org.springframework.security.core.context.SecurityContextHolder
import org.springframework.security.crypto.password.PasswordEncoder
import org.springframework.stereotype.Controller
import org.springframework.ui.Model
import org.springframework.validation.BindingResult
import org.springframework.web.bind.annotation.*
import org.springframework.web.servlet.ModelAndView
import org.springframework.web.servlet.mvc.support.RedirectAttributes


@Controller
@RequestMapping
class BugController(
    private val userRepository: UserRepository,
    private val listBugRepository: ListBugRepository,
    private val projectRepository: ProjectRepository,
    private val platformRepository: PlatformRepository,
    private val roleRepository: RoleRepository,
    private val bugService: BugService,
    private val userService: UserService,
    private val projectService: ProjectService,
    private val passwordEncoder: PasswordEncoder
) {
    val qcResultType = Status.QCResultType.values()
    val worList = Status.WorkedType.values()
    val wStList = Status.WorkedStatusType.values()
    val dStList = Status.DevStatusType.values()
    var global:String = ""

    private fun getProject(): MutableList<Project> {
        return this.projectRepository.findAll()
    }

    private fun getPlatform(): MutableList<Platform> {
        return this.platformRepository.findAll()
    }

    private fun getUser(): MutableList<User> {
        return this.userRepository.findAll()
    }

    private fun getRole(): MutableList<Role> {
        return this.roleRepository.findAll()
    }

    @GetMapping(value = ["/login"])
    fun showLoginForm(model: Model): String {
        val authentication: Authentication? = SecurityContextHolder.getContext().authentication
        return if (authentication == null || authentication is AnonymousAuthenticationToken) {
            "login"
        } else "redirect:/proj"
    }

    @GetMapping(value = ["/reg"])
    fun showRegister(model: Model): String {
        val userRole = UserRole()
        userRole.oldPass = "***"
        model.addAttribute("userRole", userRole)
        model.addAttribute("role", getRole())
        return "reg"
    }

    @PostMapping(value = ["/reg"])
    fun saveRegister(
        @Valid @ModelAttribute("userRole") userRole: UserRole, bindingResult: BindingResult,
        redirectAttributes: RedirectAttributes, model: Model,
    ): String {
        val newUser = User()
        newUser.username = userRole.username
        newUser.password = passwordEncoder.encode(userRole.password)
        newUser.roles = userRole.role
        newUser.isEnabled = true
        return if (bindingResult.hasErrors()) {
            model.addAttribute("role", getRole())
            "reg"
        } else {
            userRepository.save(newUser)
            redirectAttributes.addFlashAttribute("successMessage", "Success save User")
            "redirect:/login"
        }
    }

    @RequestMapping(value = ["/userList"])
    fun viewUser(model: Model): String {
        val myUserDetails = SecurityContextHolder.getContext().authentication.name
        val userLogin = userRepository.findByUsername(myUserDetails)
        val user2 = userService[userLogin.get().id]
        model.addAttribute("user", getUser())
        model.addAttribute("user2", user2)
        return "userList"
    }

    @RequestMapping(value = ["/proj"])
    fun viewProject(model: Model): String {
        model.addAttribute("project", getProject())
        return "projList"
    }

    @GetMapping(value = ["/projNew"])
    fun addProject(model: Model): String {
        val project = Project()
        model.addAttribute("project", project)
        return "add_proj"
    }

    @PostMapping(value = ["/projNew"])
    fun saveProject(
        @Valid @ModelAttribute("project") project: Project,
        bindingResult: BindingResult,
        redirectAttributes: RedirectAttributes,
        model: Model,
    ): String {
        return if (bindingResult.hasErrors()) {
            "add_proj"
        } else {
            projectRepository.save(project)
            redirectAttributes.addFlashAttribute("successMessage", "Success save project")
            return "redirect:/proj"
        }
    }

    @RequestMapping(value = ["/index/{proj}"])
    fun viewHomePage(
        model: Model,
        pForm: String? = "",
        programmer: String? = "",
        @PathVariable(name = "proj") proj: String,
        request: HttpServletRequest
    ): String {
        val listBugs = this.listBugRepository.done(pForm ?: "", programmer ?: "", proj).sortedBy { it.date }
        val url = request.requestURL.split("/")
        model.addAttribute("project", url.last())
        model.addAttribute("platform", getPlatform())
        model.addAttribute("user", getUser())
        model.addAttribute("listBugs", listBugs)
        return "index"
    }

    @RequestMapping(value = ["/inPro/{proj}"])
    fun viewInProgress(
        model: Model,
        pForm: String? = "",
        programmer: String? = "",
        @PathVariable(name = "proj") proj: String,
        request: HttpServletRequest
    ): String {
        val listBugs = this.listBugRepository.inPro(pForm ?: "", programmer ?: "", proj).sortedBy { it.date }
        val url = request.requestURL.split("/")
        model.addAttribute("project", url.last())
        model.addAttribute("platform", getPlatform())
        model.addAttribute("user", getUser())
        model.addAttribute("listBugs", listBugs)
        return "inPro"
    }

    @GetMapping(value = ["/new"])
    fun showNewBugForm(model: Model): String {
        val bug = ListBug()
        model.addAttribute("bug", bug)
        model.addAttribute("user", getUser())
        model.addAttribute("platform", getPlatform())
        model.addAttribute("qcResultType", qcResultType)
        return "add_bug"
    }

    @PostMapping(value = ["/new"])
    fun submitNewBugForm(
        @Valid @ModelAttribute("bug") bug: ListBug,
        bindingResult: BindingResult,
        redirectAttributes: RedirectAttributes,
        model: Model
    ): String {
        if (bug.qcId == null) {
            val myUserDetails = SecurityContextHolder.getContext().authentication.name
            val userLogin = userRepository.findByUsername(myUserDetails)
            if (userLogin.isPresent) {
                bug.qcId = userLogin.get()
            }
        }
        return if (bindingResult.hasErrors()) {
            model.addAttribute("user", getUser())
            model.addAttribute("platform", getPlatform())
            model.addAttribute("qcResultType", qcResultType)
            "add_bug"
        } else {
            bugService.save(bug)
            val project = this.projectRepository.findProjectById(bug.id.toString())
            redirectAttributes.addFlashAttribute("successMessage", "Success save bug")
            "redirect:/inPro/$project"
        }
    }

    @PostMapping(value = ["/save"])
    fun saveBug(
        @Valid @ModelAttribute("bug") bug: ListBug,
        bindingResult: BindingResult,
        redirectAttributes: RedirectAttributes,
        httpServletRequest: HttpServletRequest,
        @RequestParam page: String,
        model: Model,
    ): String {
        if (bindingResult.hasErrors()) {
            model.addAttribute("listQc", getUser())
            model.addAttribute("listPro", getUser())
            model.addAttribute("platform", getPlatform())
            model.addAttribute("bug", bug)
            model.addAttribute("worList", worList)
            model.addAttribute("wStList", wStList)
            model.addAttribute("dStList", dStList)
            model.addAttribute("qcResultType", qcResultType)
            return "edit_bug"
        }
        bugService.save(bug)
        redirectAttributes.addFlashAttribute("successMessage", "Success save bug")
        return "redirect:/$global"
    }

    @PostMapping(value = ["/saveUser"])
    fun saveUserEdit(
        @Valid @ModelAttribute("userRole") userRole: UserRole,
        bindingResult: BindingResult,
        model: Model
    ): String {
        val existUser = userService[userRole.id]
        val editUser = User()
        editUser.id = userRole.id
        editUser.username = userRole.username
        editUser.password = passwordEncoder.encode(userRole.password)
        editUser.roles = userRole.role
        editUser.isEnabled = true
        if (bindingResult.hasErrors()) {
            return "edit_user"
        }
        userService.save(editUser)
        return "redirect:/userList"
    }

    @PostMapping(value = ["/chgPass"])
    fun saveChangePass(
        @Valid @ModelAttribute("userRole") userRole: UserRole,
        bindingResult: BindingResult,
        model: Model,
        redirectAttributes: RedirectAttributes
    ): String {
        val existUser = userService[userRole.id]
        val editUser = User()
        editUser.id = userRole.id
        editUser.username = userRole.username
        editUser.password = passwordEncoder.encode(userRole.password)
        editUser.roles = existUser.roles
        editUser.isEnabled = true
        userRole.role = existUser.roles
        if (bindingResult.hasErrors()) {
            model.addAttribute("userRole", userRole)
            model.addAttribute("role", getRole())
            return "change_pass"
        }
        if (!userService.checkIfValidOldPassword(userRole.oldPass, existUser.password)) {
            model.addAttribute("userRole", userRole)
            model.addAttribute("role", getRole())
            redirectAttributes.addFlashAttribute("successMessage", "old password not match")
            return "redirect:/userList/chgPass/" + userRole.id
        }
        userService.save(editUser)
        redirectAttributes.addFlashAttribute("successMessage", "Success save user")
        return "redirect:/userList"
    }

    @PostMapping(value = ["/saveProj"])
    fun saveProjEdit(
        @Valid @ModelAttribute("project") project: Project,
        bindingResult: BindingResult,
        model: Model,
        redirectAttributes: RedirectAttributes
    ): String {
        if (bindingResult.hasErrors()){
            return "edit_proj"
        }
        redirectAttributes.addFlashAttribute("successMessage", "Success save project")
        projectService.save(project)
        return "redirect:/proj"
    }

    @RequestMapping("/edit/{id}")
    fun showEditBugForm(
        @PathVariable(name = "id") id: Long,
        @RequestParam page: String,
    ): ModelAndView {
        global = page
        val mav = ModelAndView("edit_bug")
        val bug: ListBug = bugService[id]
        mav.addObject("listQc", getUser())
        mav.addObject("listPro", getUser())
        mav.addObject("platform", getPlatform())
        mav.addObject("bug", bug)
        mav.addObject("worList", worList)
        mav.addObject("wStList", wStList)
        mav.addObject("dStList", dStList)
        mav.addObject("qcResultType", qcResultType)
        return mav
    }

    @RequestMapping("/userList/edit/{id}")
    fun editUser(@PathVariable(name = "id") id: Long): ModelAndView {
        val mav = ModelAndView("edit_user")
        val user: User = userService[id]
        val userRole = UserRole()
        userRole.id = user.id
        userRole.username = user.username
        userRole.password = user.password
        userRole.role = user.roles
        userRole.oldPass = "***"
        mav.addObject("userRole", userRole)
        mav.addObject("role", getRole())
        return mav
    }

    @RequestMapping("/userList/chgPass/{id}")
    fun changePass(@PathVariable(name = "id") id: Long): ModelAndView {
        val mav = ModelAndView("change_pass")
        val user: User = userService[id]
        val userRole = UserRole()
        userRole.id = user.id
        userRole.username = user.username
        userRole.password = user.password
        userRole.role = user.roles
        mav.addObject("userRole", userRole)
        mav.addObject("role", getRole())
        return mav
    }

    @RequestMapping("/proj/edit/{id}")
    fun editProject(@PathVariable(name = "id") id: Long): ModelAndView {
        val mav = ModelAndView("edit_proj")
        val project: Project = projectService[id]
        mav.addObject("project", project)
        return mav
    }

    @RequestMapping("/userList/delete/{id}")
    fun deleteUser(@PathVariable(name = "id") id: Long): String {
        val user = userService[id]
        userService.delete(user)
        return "redirect:/userList"
    }

    @RequestMapping("/proj/delete/{id}")
    fun deleteProj(@PathVariable(name = "id") id: Long): String {
        projectService[id]
        projectService.delete(id)
        return "redirect:/proj"
    }

    @RequestMapping("/delete/{id}")
    fun deleteBug(@PathVariable(name = "id") id: Long): String {
        val project = this.projectRepository.findProjectById(id.toString())
        bugService[id]
        bugService.delete(id)
        return "redirect:/index/$project"
    }

    @RequestMapping("/delete2/{id}")
    fun delete2Bug(@PathVariable(name = "id") id: Long): String {
        val project = this.projectRepository.findProjectById(id.toString())
        bugService[id]
        bugService.delete(id)
        return "redirect:/inPro/$project"
    }

    @RequestMapping("/")
    fun redirect(): String {
        return "redirect:/proj"
    }

}

